#include "stdafx.h"
#include "functions_usb.h"
#include "functions_jw24f14.h"




//sensitivitiy LSB/g
//#define SENSITIVITIY_LSB_G 2730

//#define CONST_G 9.80665
 //convert to m/s^2
 //       xAcc = ((float) xAcc_raw / SENSITIVITIY_LSB_G) * CONST_G;

bool jw24f14_write(HANDLE handle, BYTE command, BYTE address, BYTE data)
{
	UCHAR			WriteBuffer[9];
	UCHAR			ReadBuffer[9];
	BOOL			Result;
	DWORD			BytesWritten = 0;
	DWORD			NumberOfBytesRead = 0;
	HIDP_CAPS		caps;

	if(handle == 0)
		return 1;	//Error No Handle

	caps = GetPreparsedData(handle);

	memset(&WriteBuffer, 0, caps.OutputReportByteLength);
	WriteBuffer[0] = 0x00;
	WriteBuffer[1] = command;
	WriteBuffer[2] = address;
	WriteBuffer[3] = data;

	Result = WriteFile(handle, &WriteBuffer, caps.OutputReportByteLength, &BytesWritten, NULL);

	if(Result != NULL)
	{
		
		memset(&ReadBuffer, 0, caps.InputReportByteLength);
		ReadBuffer[0] = 0x00;
	
		Result = ReadFile(handle, &ReadBuffer, caps.InputReportByteLength, &NumberOfBytesRead, NULL);

		if(Result != NULL)
			return 0;	//Success
		else
			return 1;	//Read Error
	}
	else
		return 1;	//Write Error
}


BYTE jw24f14_read(HANDLE handle, BYTE address)
{
	UCHAR			WriteBuffer[9];
	UCHAR			ReadBuffer[9];
	DWORD			BytesWritten = 0;
	DWORD			NumberOfBytesRead = 0;
	BOOL			Result;
	HIDP_CAPS		caps;

	if(handle == 0)
		return 1;	//Error No Handle

	caps = GetPreparsedData(handle);

	HidD_FlushQueue(handle);

	memset(&WriteBuffer, 0, caps.OutputReportByteLength);

	WriteBuffer[0] = 0x00; //ReportID
	WriteBuffer[1] = 0x82; //CMD-Mode
	WriteBuffer[2] = address | 0x80; //CMD + Addr

	Result = WriteFile(handle, &WriteBuffer, caps.OutputReportByteLength, &BytesWritten, NULL);

	if(Result != NULL)
	{
		memset(&ReadBuffer, 0, caps.InputReportByteLength);
		ReadBuffer[0] = 0x00;
	
		ReadFile(handle, &ReadBuffer, caps.InputReportByteLength, &NumberOfBytesRead, NULL);
			return ReadBuffer[3];
	}
	else
		return 0;
}

//Enable or disable direct connection
void jw24f14_mode(HANDLE handle, bool status)
{
	switch(status){
	case 1:
		jw24f14_write(handle, 0x82, 0x00, 0x00);
		break;
	case 0:
		jw24f14_write(handle, 0x00, 0x00, 0x00);
		break;
	}
}

//Set image writeable
void jw24f14_open_image(HANDLE handle, bool status)
{

	BYTE state;

	state = jw24f14_read(handle, 0x0D) & 0xEF;

	switch(status)
	{
	case 1:
		jw24f14_write(handle, 0x82, 0x0D, 0x10 | state);
		break;
	case 0:
		jw24f14_write(handle, 0x82, 0x0D, 0x00 | state);
		break;
	}
}

void jw24f14_write_ee(HANDLE handle, BYTE address)
{
	UCHAR			WriteBuffer[10];
	UCHAR			ReadBuffer[10];
	BOOL			Result;
	DWORD			BytesWritten = 0;
	DWORD			NumberOfBytesRead = 0;
	HIDP_CAPS		caps;

	caps = GetPreparsedData(handle);

	memset(&WriteBuffer, 0, caps.OutputReportByteLength+1);
	WriteBuffer[0] = 0x00;
	WriteBuffer[1] = 0x82;
	WriteBuffer[2] = address & 0xFE;

	Result = WriteFile(handle, &WriteBuffer, caps.OutputReportByteLength, &BytesWritten, NULL);

	memset(&ReadBuffer, 0, caps.InputReportByteLength+1);
	ReadBuffer[0] = 0x00;
	
	ReadFile(handle, &ReadBuffer, caps.InputReportByteLength, &NumberOfBytesRead, NULL);
}

void jw24f14_get_image(HANDLE handle, BYTE image[60])
{

	for(int i=0; i<60; i++)
	{
		image[i] = jw24f14_read(handle, i);
	}

}

void jw24f14_softreset(HANDLE handle)
{
	jw24f14_open_image(handle, true);
	jw24f14_write(handle, 0x82, 0x10, 0xB6);
}

void jw24f14_update_image(HANDLE handle)
{
	jw24f14_write(handle, 0x82, 0x0D, 0x20);
}

void jw24f14_reset_int(HANDLE handle)
{
	BYTE state;

	state = jw24f14_read(handle, 0x0D);
	jw24f14_write(handle, 0x82, 0x0D, 0x40 | state);
}

short jw24f14_read_x(HANDLE handle)
{
	BYTE lsb, msb;
	WORD acc;

	lsb = jw24f14_read(handle, 0x02);
	msb = jw24f14_read(handle, 0x03);

	acc = ((msb << 8) | lsb) >> 2;

	return acc;
}

short jw24f14_read_y(HANDLE handle)
{
	BYTE lsb, msb;
	WORD acc;

	lsb = jw24f14_read(handle, 0x04);
	msb = jw24f14_read(handle, 0x05);

	acc = ((msb << 8) | lsb) >> 2;

	return acc;
}

short jw24f14_read_z(HANDLE handle)
{
	BYTE lsb, msb;
	WORD acc;

	lsb = jw24f14_read(handle, 0x06);
	msb = jw24f14_read(handle, 0x07);

	acc = ((msb << 8) | lsb) >> 2;

	return acc;
}

short jw24f14_read_t(HANDLE handle)
{
	return jw24f14_read(handle, 0x08);
}

bool jw24f14_read_acc_raw(HANDLE handle, unsigned short &x, unsigned short &y, unsigned short &z)
{
	UCHAR			WriteBuffer[9];
	UCHAR			ReadBuffer[9];
	DWORD			BytesWritten = 0;
	DWORD			NumberOfBytesRead = 0;
	BOOL			Result = 1;
	HIDP_CAPS		caps;

	if(handle == 0)
		return 1;	//Error No Handle

	caps = GetPreparsedData(handle);	
	HidD_FlushQueue(handle);

	memset(&WriteBuffer, 0, caps.OutputReportByteLength);

	WriteBuffer[0] = 0x00; //ReportID
	WriteBuffer[1] = 0x87; //CMD-Mode + 6 Byte Read
	WriteBuffer[2] = 0x02 | 0x80; //CMD + Addr

	Result = WriteFile(handle, &WriteBuffer, caps.OutputReportByteLength, &BytesWritten, NULL);

	if(Result != NULL)
	{
		memset(&ReadBuffer, 0x00, caps.OutputReportByteLength);
		Result = ReadFile(handle, &ReadBuffer, caps.OutputReportByteLength, &NumberOfBytesRead, NULL);

		if(Result != NULL)
		{
			x = ((ReadBuffer[4] << 8) | ReadBuffer[3]) >> 2;
			y = ((ReadBuffer[6] << 8) | ReadBuffer[5]) >> 2;
			z = ((ReadBuffer[8] << 8) | ReadBuffer[7]) >> 2;
	
			return 0;	//Success
		}
		else
			return 1;	//Error Read
	}
	else
		return 1;	//Error Write
}

bool jw24f14_read_acc_int(HANDLE handle, short &x, short &y, short &z)
{
	bool ret;
	
	unsigned short acc_x;
	unsigned short acc_y;
	unsigned short acc_z;
	
	ret = jw24f14_read_acc_raw(handle, acc_x, acc_y, acc_z);

	if(ret == 0)
	{
		//Get X
		if(acc_x & 0x2000) x = (short) acc_x - 0x3FFF;
		else x = acc_x;

		//Get Y
		if(acc_y & 0x2000) y = (short) acc_y - 0x3FFF;
		else y = acc_y;

		//Get Z
		if(acc_z & 0x2000) z = (short) acc_z - 0x3FFF;
		else z = acc_z;

		return 1;
	}
		else
			return ret;
}


BYTE jw24f14_get_range(HANDLE handle)
{
	BYTE sub = jw24f14_read(handle, 0x35) & 0x0E;
	BYTE ret = sub >> 1;

	return ret;
}

BYTE jw24f14_get_band(HANDLE handle)
{
	BYTE sub = jw24f14_read(handle, 0x20) & 0xF0;
	BYTE ret = sub >> 4;

	return ret;
}

CString jw24f14_get_range_str(BYTE range)
{
	switch(range)
	{
		case 0: return _T("1G");
		case 1: return _T("1.5G");
		case 2: return _T("2G");
		case 3: return _T("3G");
		case 4: return _T("4G");
		case 5: return _T("8G");
		case 6: return _T("16G");
		default: return _T("NULL");
	}
}

CString jw24f14_get_band_str(BYTE band)
{
	switch(band)
	{
		case 0: return _T("10 Hz");
		case 1: return _T("20 Hz");
		case 2: return _T("40 Hz");
		case 3: return _T("75 Hz");
		case 4: return _T("150 Hz");
		case 5: return _T("300 Hz");
		case 6: return _T("600 Hz");
		case 7: return _T("1200 Hz");
		case 8: return _T("High-pass");
		case 9: return _T("Band-pass");
		default: return _T("NULL");
	}
}

short jw24f14_get_offset(HANDLE handle)
{
	if(handle == 0)
		return -1;

	BYTE range = jw24f14_get_range(handle);

	switch(range)
	{
		case RANGE_1G: return OFFSET_1G ;
		case RANGE_1_5G: return OFFSET_1_5G ;
		case RANGE_2G: return OFFSET_2G ;
		case RANGE_3G: return OFFSET_3G ;
		case RANGE_4G: return OFFSET_4G ;
		case RANGE_8G: return OFFSET_8G ;
		case RANGE_16G: return OFFSET_16G ;
		default: return 0;
	}

}

void jw24f14_set_range(HANDLE handle, BYTE value)
{
	BYTE read = jw24f14_read(handle, 0x35) & 0xF1;
	jw24f14_open_image(handle, true);							//ee_w is locked to prevent mal-function. unlock for image and eeprom
	jw24f14_write(handle, 0x82, 0x35, (value << 1) | read);
	jw24f14_open_image(handle, false);
}

void jw24f14_set_band(HANDLE handle, BYTE value)
{
	BYTE read = jw24f14_read(handle, 0x20) & 0x0F;
	jw24f14_open_image(handle, true);
	jw24f14_write(handle, 0x82, 0x20, (value << 4) | read);
	jw24f14_open_image(handle, false);
}

BYTE jw24f14_get_msb_offset(HANDLE handle, BYTE axis)
{
	BYTE read = 0x00;

	//0x00: X
	//0x01: Y
	//0x02: Z

	switch(axis)
	{
	case 0x00:
		read = jw24f14_read(handle, 0x38);
		break;
	case 0x01:
		read = jw24f14_read(handle, 0x39);
		break;
	case 0x02:
		read = jw24f14_read(handle, 0x34);
		break;
	}




	return read;
}

BYTE jw24f14_get_lsb_offset(HANDLE handle, BYTE axis)
{
	BYTE read = 0x00;

	//0x00: X
	//0x01: Y
	//0x02: Z

	switch(axis)
	{
	case 0x00:
		read = jw24f14_read(handle, 0x35);
		read = (read & 0xF0) >> 4;
		
		break;
	case 0x01:
		read = jw24f14_read(handle, 0x36);
		read = read & 0x0F;

		break;
	case 0x02:
		read = jw24f14_read(handle, 0x36);
		read = (read & 0xF0) >> 4;

		break;
	}


	return read;
}