/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
* JoyWarrior24F14 functions
* Created by Code Mercenaries GmbH
* Author: Christoph Jung (jung@codemercs.com)
* http://www.codemercs.com 
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


#define MD_CMD true
#define MD_FREE false

//Endpoints of JoyWarrior
#define PIPE_JOYSTICK 0
#define PIPE_SENSOR 1

//Constant G
#define CONST_G 9.80665

//Error returns
#define NONE_ERROR 0
#define WRITE_ERROR 1
#define READ_ERROR 2
#define HANDLE_ERROR 3

//JoyWarrior24F14 product id
#define DEVICE_JW24F14_PID 0x1116

//Defines fr Register Byte Range
#define RANGE_1G 0x00
#define RANGE_1_5G 0x01
#define RANGE_2G 0x02
#define RANGE_3G 0x03
#define RANGE_4G 0x04
#define RANGE_8G 0x05
#define RANGE_16G 0x06

#define RANGE_1G_TXT _T("+/- 1.0 g")
#define RANGE_1_5G_TXT _T("+/- 1.5 g")
#define RANGE_2G_TXT _T("+/- 2.0 g")
#define RANGE_3G_TXT _T("+/- 3.0 g")
#define RANGE_4G_TXT _T("+/- 4.0 g")
#define RANGE_8G_TXT _T("+/- 8.0 g")
#define RANGE_16G_TXT _T("+/- 16.0 g")

//Defines fr Auflsung der milli G
#define RANGE_1G_mg 0.122
#define RANGE_1_5G_mg 0.183
#define RANGE_2G_mg 0.244
#define RANGE_3G_mg 0.366
#define RANGE_4G_mg 0.488
#define RANGE_8G_mg 0.976
#define RANGE_16G_mg 1.953


//Defines fr Umrechnung der Joystickdaten in milli G
#define CALC_1G 8.192
#define CALC_1_5G 5.461
#define CALC_2G 4.096
#define CALC_3G 2.731
#define CALC_4G 2.048
#define CALC_8G 1.024
#define CALC_16G 0.512

#define OFFSET_1G 8192
#define OFFSET_1_5G 5461
#define OFFSET_2G 4096
#define OFFSET_3G 2731
#define OFFSET_4G 2048
#define OFFSET_8G 1024
#define OFFSET_16G 512

//Defines fr Register Bytes Bandwidth
#define BAND_10HZ 0x00
#define BAND_20HZ 0x01
#define BAND_40HZ 0x02
#define BAND_75HZ 0x03
#define BAND_150HZ 0x04
#define BAND_300HZ 0x05
#define BAND_600HZ 0x06
#define BAND_1200HZ 0x07
#define BAND_HIGHPASS 0x08
#define BAND_BANDPASS 0x09

#define BAND_10HZ_TXT _T("10 Hz")
#define BAND_20HZ_TXT _T("20 Hz")
#define BAND_40HZ_TXT _T("40 Hz")
#define BAND_75HZ_TXT _T("75 Hz")
#define BAND_150HZ_TXT _T("150 Hz")
#define BAND_300HZ_TXT _T("300 Hz")
#define BAND_600HZ_TXT _T("600 Hz")
#define BAND_1200HZ_TXT _T("1200 Hz")
#define BAND_HIGHPASS_TXT _T("Highpass")
#define BAND_BANDPASS_TXT _T("Bandpass")

typedef struct f14_acc
{
	BYTE msb_x;
	BYTE msb_y;
	BYTE msb_z;
	BYTE msb_t;
	BYTE lsb_x;
	BYTE lsb_y;
	BYTE lsb_z;
	unsigned short acc_x;
	unsigned short acc_y;
	unsigned short acc_z;
} f14_acc;

typedef struct f14_setting
{
	BYTE range;
	BYTE bw;
	short offset;

} f14_setting;


bool jw24f14_write(HANDLE handle, BYTE command, BYTE address, BYTE data);
BYTE jw24f14_read(HANDLE handle, BYTE address);
void jw24f14_write_ee(HANDLE handle, BYTE address);

void jw24f14_mode(HANDLE handle, bool status);
void jw24f14_open_image(HANDLE handle, bool status);
void jw24f14_get_image(HANDLE handle, BYTE image[60]);
void jw24f14_softreset(HANDLE handle);
void jw24f14_update_image(HANDLE handle);
void jw24f14_reset_int(HANDLE handle);

short jw24f14_read_x(HANDLE handle);
short jw24f14_read_y(HANDLE handle);
short jw24f14_read_z(HANDLE handle);
short jw24f14_read_t(HANDLE handle);

bool jw24f14_read_acc_raw(HANDLE handle, unsigned short &x, unsigned short &y, unsigned short &z);
bool jw24f14_read_acc_int(HANDLE handle, short &x, short &y, short &z);


void jw24f14_set_range(HANDLE handle, BYTE range);
void jw24f14_set_band(HANDLE handle, BYTE band);
BYTE jw24f14_get_range(HANDLE handle);
BYTE jw24f14_get_band(HANDLE handle);
CString jw24f14_get_range_str(BYTE range);
CString jw24f14_get_band_str(BYTE band);

short jw24f14_get_offset(HANDLE handle);

BYTE jw24f14_get_msb_offset(HANDLE handle, BYTE axis);
BYTE jw24f14_get_lsb_offset(HANDLE handle, BYTE axis);
