#include "stdafx.h"
#include "math.h"
#include "functions_usb.h"


CString GetSerialNumber(HANDLE handle)
{
	WCHAR sn[32];
	CString	serial;
	
	memset(&sn, 0, 32);
	HidD_GetSerialNumberString(handle, &sn, 32); 

	serial.Format(_T("%s"), &sn[0]);
	return serial;
}

HIDP_CAPS GetPreparsedData(HANDLE handle)
{
	PHIDP_PREPARSED_DATA	PreparsedData;
	HIDP_CAPS				Caps;

	HidD_GetPreparsedData(handle, &PreparsedData);
	HidP_GetCaps(PreparsedData, &Caps);
	HidD_FreePreparsedData(PreparsedData);

	return Caps;
}

int EnumHidDevices(ULONG productID, ULONG vendorID, CString devSerial[32])
{
	HIDD_ATTRIBUTES						Attributes;
	SP_DEVICE_INTERFACE_DATA			devInfoData;
	PSP_DEVICE_INTERFACE_DETAIL_DATA	detailData;
	HDEVINFO							hDevInfo;
	int									MemberIndex = 0;
	LONG								DevInfo;
	ULONG								Length = 0;
	ULONG								Required;
	GUID								HidGuid;
	HANDLE								devHandle;
	CString								serial;
	int									counter = 0;

    ZeroMemory(&devInfoData, sizeof(devInfoData));
    devInfoData.cbSize = sizeof(devInfoData);

	HidD_GetHidGuid(&HidGuid);	

	hDevInfo = SetupDiGetClassDevsW(&HidGuid, NULL, NULL, DIGCF_DEVICEINTERFACE | DIGCF_PRESENT);

	do{
		DevInfo = SetupDiEnumDeviceInterfaces (hDevInfo, NULL, &HidGuid, MemberIndex, &devInfoData);

		if (DevInfo != 0)
		{
			SetupDiGetDeviceInterfaceDetailW(hDevInfo, &devInfoData, NULL, 0, &Length, NULL);

			detailData = (PSP_DEVICE_INTERFACE_DETAIL_DATA) malloc(Length);
			detailData->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);

			SetupDiGetDeviceInterfaceDetailW(hDevInfo, &devInfoData, detailData, Length, &Required, NULL);

			devHandle = CreateFileW(detailData->DevicePath, 0, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_FLAG_NO_BUFFERING, NULL);

			Attributes.Size = sizeof(Attributes);
			HidD_GetAttributes (devHandle, &Attributes);

			if((Attributes.VendorID == vendorID) && (Attributes.ProductID == productID))
			{
				/*Generate list of serials*/
				/*****************************************************************************/
				serial.SetString(GetSerialNumber(devHandle));

				for(int i=0; i<32; i++)
					if((devSerial[i] == serial) || (serial.GetLength() < 1))
						counter++;

				if(counter <= 0)
					devSerial[MemberIndex].SetString(serial);

				/*****************************************************************************/
			}

		CloseHandle(devHandle);

		free(detailData);
		}

		MemberIndex++;

	}while (DevInfo != NULL);

	SetupDiDestroyDeviceInfoList(hDevInfo);

	return 0;
}

BOOL ConnectToUSB(ULONG productID, ULONG vendorID, HANDLE &devHandle, HANDLE hidHandle[4], CString &devSerial, HIDP_CAPS &Caps)
{
	HIDD_ATTRIBUTES						Attributes;
	SP_DEVICE_INTERFACE_DATA			devInfoData;
	PSP_DEVICE_INTERFACE_DETAIL_DATA	detailData;
	HDEVINFO							hDevInfo;
	int									MemberIndex = 0;
	int									DeviceIndex = 0;
	LONG								DevInfo;
	ULONG								Length = 0;
	ULONG								Required;
	GUID								HidGuid;
	HANDLE								dHandle;

    ZeroMemory(&devInfoData, sizeof(devInfoData));
    devInfoData.cbSize = sizeof(devInfoData);

	HidD_GetHidGuid(&HidGuid);	

	hDevInfo = SetupDiGetClassDevsW(&HidGuid, NULL, NULL, DIGCF_DEVICEINTERFACE | DIGCF_PRESENT);

	do{
		DevInfo = SetupDiEnumDeviceInterfaces (hDevInfo, NULL, &HidGuid, MemberIndex, &devInfoData);

		if (DevInfo != 0)
		{
			SetupDiGetDeviceInterfaceDetailW(hDevInfo, &devInfoData, NULL, 0, &Length, NULL);

			detailData = (PSP_DEVICE_INTERFACE_DETAIL_DATA) malloc(Length);
			detailData->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);

			SetupDiGetDeviceInterfaceDetailW(hDevInfo, &devInfoData, detailData, Length, &Required, NULL);

			dHandle = CreateFileW(detailData->DevicePath, 0, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_FLAG_NO_BUFFERING, NULL);
			
			Attributes.Size = sizeof(Attributes);
			HidD_GetAttributes (dHandle, &Attributes);

			if((Attributes.VendorID == vendorID) && (Attributes.ProductID == productID))
			{
				if((GetSerialNumber(dHandle) == devSerial) && (devSerial.GetLength() > 0))
				{
					Caps = GetPreparsedData(dHandle);
					devHandle = dHandle;
					
					hidHandle[DeviceIndex] = CreateFileW(detailData->DevicePath, 
														GENERIC_WRITE | GENERIC_READ, 
														FILE_SHARE_READ|FILE_SHARE_WRITE, 
														NULL,
														OPEN_EXISTING, 
														FILE_FLAG_NO_BUFFERING, 
														NULL);
					
					DeviceIndex++;
				}
			}

		CloseHandle(devHandle);
			
		free(detailData);
		}

		MemberIndex++;

	}while (DevInfo != NULL);

	SetupDiDestroyDeviceInfoList(hDevInfo);

	return 0;
}

BOOL ConnectToUSB2(ULONG productID, ULONG vendorID, HANDLE hidHandle[4], CString &devSerial, HIDP_CAPS &Caps)
{
	HIDD_ATTRIBUTES						Attributes;
	SP_DEVICE_INTERFACE_DATA			devInfoData;
	PSP_DEVICE_INTERFACE_DETAIL_DATA	detailData;
	HDEVINFO							hDevInfo;
	int									MemberIndex = 0;
	int									DeviceIndex = 0;
	LONG								DevInfo;
	ULONG								Length = 0;
	ULONG								Required;
	GUID								HidGuid;
	HANDLE								dHandle = NULL;
	BOOL								stat = FALSE;

	ZeroMemory(&devInfoData, sizeof(devInfoData));
	devInfoData.cbSize = sizeof(devInfoData);

	HidD_GetHidGuid(&HidGuid);	
	hDevInfo = SetupDiGetClassDevsW(&HidGuid, NULL, NULL, DIGCF_DEVICEINTERFACE | DIGCF_PRESENT);

	do{
		DevInfo = SetupDiEnumDeviceInterfaces (hDevInfo, NULL, &HidGuid, MemberIndex, &devInfoData);

		if (DevInfo != 0){
			SetupDiGetDeviceInterfaceDetailW(hDevInfo, &devInfoData, NULL, 0, &Length, NULL);

			detailData = (PSP_DEVICE_INTERFACE_DETAIL_DATA) malloc(Length);
			detailData->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);

			SetupDiGetDeviceInterfaceDetailW(hDevInfo, &devInfoData, detailData, Length, &Required, NULL);

			dHandle = CreateFileW(detailData->DevicePath, 0, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_FLAG_NO_BUFFERING, NULL);

			if(dHandle != NULL){
				Attributes.Size = sizeof(Attributes);
				HidD_GetAttributes (dHandle, &Attributes);

				if((Attributes.VendorID == vendorID) && (Attributes.ProductID == productID)){
					if((GetSerialNumber(dHandle) == devSerial) && (devSerial.GetLength() > 0)){
						Caps = GetPreparsedData(dHandle);

						hidHandle[DeviceIndex] = CreateFileW(detailData->DevicePath, 
							GENERIC_WRITE | GENERIC_READ, 
							FILE_SHARE_READ|FILE_SHARE_WRITE, 
							NULL,
							OPEN_EXISTING, 
							FILE_FLAG_NO_BUFFERING, 
							NULL);

						DeviceIndex++;
					}
				}
				stat = TRUE;
				CloseHandle(dHandle);
			}
			free(detailData);
		}

		MemberIndex++;

	}while (DevInfo != NULL);

	SetupDiDestroyDeviceInfoList(hDevInfo);

	return stat;
}

BOOL WriteData2(HANDLE handle, HIDP_CAPS Caps, UCHAR WBuffer[10], UCHAR RBuffer[10])
{
	BOOL			Result;
	DWORD			BytesWritten = 0;
	DWORD			NumberOfBytesRead = 0;

	memset(&WBuffer, 0, Caps.OutputReportByteLength+1);

	//WriteBuffer[0] = repID;
	//WriteBuffer[1...9] = cmd;

	Result = WriteFile(handle, &WBuffer, Caps.OutputReportByteLength, &BytesWritten, NULL);

	if(Result != NULL)
	{
		memset(&RBuffer, 0, Caps.InputReportByteLength+1);
		//ReadBuffer[0] = 0x00;
	
		ReadFile(handle, &RBuffer, Caps.InputReportByteLength, &NumberOfBytesRead, NULL);

		return 1;
	}
	else
		return 0;
}

BOOL ReadData2(HANDLE handle, HIDP_CAPS Caps, UCHAR WBuffer[10], UCHAR RBuffer[10])
{
	DWORD			BytesWritten = 0;
	DWORD			NumberOfBytesRead = 0;
	BOOL			Result;

	HidD_FlushQueue(handle);
	//newAddr = 0x80 | addr;

	memset(&WBuffer, 0, Caps.OutputReportByteLength+1);

	/*Enable command-mode from Jw*/
	//WriteBuffer[0] = 0x00; //ReportID
	//WriteBuffer[1] = 0x82; //CMD-Mode
	//WriteBuffer[2] = newAddr; //CMD + Addr

	Result = WriteFile(handle, &WBuffer, Caps.OutputReportByteLength, &BytesWritten, NULL);

	if(Result != NULL)
	{
		memset(&RBuffer, 0, Caps.InputReportByteLength+1);
		//RBuffer[0] = 0x00;
	
		ReadFile(handle, &RBuffer, Caps.InputReportByteLength, &NumberOfBytesRead, NULL);
		return 1;
	}
	else
		return 0;
}

/*USB-Function: Read data from interface 0*/
BOOL ReadPipOne2(HANDLE handle, HIDP_CAPS Caps, BYTE *RBuffer)
{
	DWORD NumberOfBytesRead = 0;

	ReadFile(handle, RBuffer, Caps.InputReportByteLength, &NumberOfBytesRead, NULL);
	
	return 1;
}

int EnumAllHidDevices(ULONG productID, ULONG vendorID, CString devSerial[32], ULONG devPid[32], CStringArray *SerialArray)
{
	HIDD_ATTRIBUTES						Attributes;
	SP_DEVICE_INTERFACE_DATA			devInfoData;
	PSP_DEVICE_INTERFACE_DETAIL_DATA	detailData;
	HDEVINFO							hDevInfo;
	int									MemberIndex = 0;
	LONG								DevInfo;
	ULONG								Length = 0;
	ULONG								Required;
	GUID								HidGuid;
	HANDLE								devHandle;
	//CString							serial;
	//int								counter = 0;
	BOOL								bDev = FALSE;
	//CStringArray						SerialArray;
	int									DeviceIndex = 0;

    ZeroMemory(&devInfoData, sizeof(devInfoData));
    devInfoData.cbSize = sizeof(devInfoData);

	HidD_GetHidGuid(&HidGuid);	

	hDevInfo = SetupDiGetClassDevsW(&HidGuid, NULL, NULL, DIGCF_DEVICEINTERFACE | DIGCF_PRESENT);

	do{
		DevInfo = SetupDiEnumDeviceInterfaces (hDevInfo, NULL, &HidGuid, MemberIndex, &devInfoData);

		if (DevInfo != 0)
		{
			SetupDiGetDeviceInterfaceDetailW(hDevInfo, &devInfoData, NULL, 0, &Length, NULL);

			detailData = (PSP_DEVICE_INTERFACE_DETAIL_DATA) malloc(Length);
			detailData->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);

			SetupDiGetDeviceInterfaceDetailW(hDevInfo, &devInfoData, detailData, Length, &Required, NULL);

			devHandle = CreateFileW(detailData->DevicePath, 0, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_FLAG_NO_BUFFERING, NULL);

			Attributes.Size = sizeof(Attributes);
			HidD_GetAttributes (devHandle, &Attributes);

			if(Attributes.VendorID == vendorID)
			{
				SerialArray->Add(GetSerialNumber(devHandle));
				/*Generate list of serials*/
				/*****************************************************************************/
				for(int i=0; i<32; i++)
					if (devSerial[i] == GetSerialNumber(devHandle))
						bDev = TRUE;
				
				if(bDev == FALSE)
				{
					devSerial[DeviceIndex] = GetSerialNumber(devHandle);
					devPid[DeviceIndex] = Attributes.ProductID;
				}
				/*****************************************************************************/

				DeviceIndex++;
			}

			CloseHandle(devHandle);
			bDev = FALSE;
			
			free(detailData);
		}

		MemberIndex++;

	}while (DevInfo != NULL);

	SetupDiDestroyDeviceInfoList(hDevInfo);

	return 0;
}

BOOL DevicePresent(HANDLE handle, HIDP_CAPS Caps)
{
	BYTE Buffer[1];
	ULONG NumberOfBytesRead = 0;
	BOOL Result;



	//HIDP_CAPS caps = GetPreparsedData(handle);

	//Result = HidD_GetInputReport(handle, &Buffer, BytesWritten);

	Result = ReadFile(handle, Buffer, Caps.InputReportByteLength, &NumberOfBytesRead, NULL);
	
	//return ReadFile(handle, &Buffer, 1, &NumberOfBytesRead, NULL);
	return Result;
}